load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/contrib/calendar_decode2.ncl"
date_start = 2020072100
date_end = 2020072300
date_skip = 1
lat0 = -5.
lon0 = 300.


days = new((/1000/),float)
ndays = 0
date = date_start
do while (date.le.date_end)

ndays = ndays+1
print(""+date)
year = floattointeger(date/1.e6)
month = floattointeger((date-year*1000000)/10000.)
day = floattointeger((date-year*1000000-month*10000)/100.)
hour = date-year*1000000-month*10000-day*100
days(ndays-1) = day_of_year(year,month,day)+hour/24.
hour = hour+date_skip
if(hour.gt.23) then
 hour = hour-24
 day = day+1
end if
date = year*1000000+month*10000+day*100+hour

end do
;print("ndays = "+ndays)
;print(""+days(0:ndays-1))

strings = new((/1000/),string)

nstrings = 0
strings(nstrings) = ""+ndays
do k=1,ndays
nstrings = nstrings+1
strings(nstrings) = ""+days(k-1)
end do

date = date_start

write_table("snd", "w", [/"  z[m] p[mb] tp[K] q[g/kg] u[m/s] v[m/s]"/], "%s")

do while (date.le.date_end)

print(""+date)
year = floattointeger(date/1.e6)
month = floattointeger((date-year*1000000)/10000.)
day = floattointeger((date-year*1000000-month*10000)/100.)
hour = date-year*1000000-month*10000-day*100
print(""+(date-year*1000000-month*10000)/100.)
print(year+" "+month+" "+day+" "+hour)
print("day_of_year="+(day_of_year(year,month,day)+hour/24.))

dates_obs = (year*1000000+month*10000+day*100)+"_"+(year*1000000+month*10000+day*100+23)
month_obs = ""+(year*100+month)
;print(""+dates_obs)

dataset = "era5"
dirpath = "/glade/campaign/collections/rda/data/ds633.0/e5.oper.an.pl/"+month_obs+"/"

f_u = addfile(dirpath+"e5.oper.an.pl.128_131_u.ll025uv."+dates_obs+".nc","r")
f_v = addfile(dirpath+"e5.oper.an.pl.128_132_v.ll025uv."+dates_obs+".nc","r")
f_w = addfile(dirpath+"e5.oper.an.pl.128_135_w.ll025sc."+dates_obs+".nc","r")
f_z = addfile(dirpath+"e5.oper.an.pl.128_129_z.ll025sc."+dates_obs+".nc","r")
f_t = addfile(dirpath+"e5.oper.an.pl.128_130_t.ll025sc."+dates_obs+".nc","r")
f_q = addfile(dirpath+"e5.oper.an.pl.128_133_q.ll025sc."+dates_obs+".nc","r")
f_clwc = addfile(dirpath+"e5.oper.an.pl.128_246_clwc.ll025sc."+dates_obs+".nc","r")
f_ciwc = addfile(dirpath+"e5.oper.an.pl.128_247_ciwc.ll025sc."+dates_obs+".nc","r")

times = calendar_decode2(f_u->time,-5)
dates = times(:,0)*1000000+times(:,1)*10000+times(:,2)*100+times(:,3)
;print(""+dates)
ntime = dimsizes(f_u->time)
itime = -1
do i=0,ntime-1
 if(dates(i).eq.date) then
  if(itime.eq.-1) then
     itime = i
  end if
 end if
end do
if(itime.eq.-1) then
  print("could not find date "+date)
  exit
end if
time = day_of_year(times(itime,0),times(itime,1),times(itime,2))+times(itime,3)/24.
print("  time="+time)

nlat = dimsizes(f_u->latitude)
nlon = dimsizes(f_u->longitude)
nzi = dimsizes(f_u->level)

mmm = itime

; determine pressure levels at z:

nlati = dimsizes(f_u->latitude)
presi = doubletofloat(f_u->level(nzi-1:0:1))
z3 = f_z->Z(mmm,nzi-1:0:1,nlati-1:0:1,:)
lat = doubletofloat(z3&latitude)
lon = doubletofloat(z3&longitude)
wgt = cos(z3&latitude*acos(-1.)/180.)
z3m = dim_avg_Wrap(dim_avg_wgt_n_Wrap(z3,wgt,0,1))/9.80665
;print("z3m: "+z3m+"  presi: "+presi)

print("reading ...")
fld3=f_u->U(mmm,nzi-1:0:1,nlati-1:0:1,:)
u=fld3(:,{lat0},{lon0})
fld3=f_v->V(mmm,nzi-1:0:1,nlati-1:0:1,:)
v=fld3(:,{lat0},{lon0})
fld3=f_t->T(mmm,nzi-1:0:1,nlati-1:0:1,:)
t=fld3(:,{lat0},{lon0})
fld3=f_q->Q(mmm,nzi-1:0:1,nlati-1:0:1,:)
q=fld3(:,{lat0},{lon0})
zzz=z3(:,{lat0},{lon0})/9.81
pres0 = presi(0)+(presi(0)-presi(1))/(zzz(1)-zzz(0))*zzz(0)
tmp1=(/time/)
tmp2=(/nzi/)
tmp3=(/pres0/)
write_table("snd", "a", [/time,nzi,pres0/], "%f  %i  %f")
write_table("snd", "a", [/zzz,presi,(t*(1000./presi)^(287./1004.)),q*1.e3,u,v/],"%f  %f  %f  %f  %f  %f")
;print(zzz+" "+presi+" "+(t*(1000./presi)^(287./1004.))+" "+(q*1.e3)+" "+u+" "+v)

hour = hour+date_skip
if(hour.gt.23) then
 hour = hour-24
 day = day+1
end if
date = year*1000000+month*10000+day*100+hour

end do
