subroutine setfixdynamics()

! overwrite dynamics with prescribed fields if dosetdynamics = .true.

use grid
use vars, only: u,v,w,rho,rhow
use params, only: dofixdynamics, fixdynamics_type
implicit none

real top, x, x0, xc, zc, pi
real psi(nx+1,nzm+1)
integer i,j,k,it,jt

select case (fixdynamics_type)

 case(1)

  ! single prescribed 2D cumulus cloud

  top = 12000. ! height of cloud top
  pi = acos(-1.)

  call task_rank_to_index(rank,it,jt)

  x0 = dx*nx_gl/4.
  psi = 0.
  do k=1,nzm+1
    if(zi(k).le.top) then
     do i=1,nx+1
       x = dx*(i+it-nx_gl/2-1)/x0
       psi(i,k) = x*exp(-2.*x**2)*sin(pi*zi(k)/top)*1.e5/5. 
     end do
    end if
  end do
  if(1+it.eq.1) psi(1,:) = 0.
  if(nx+1+it.eq.nx_gl+1) psi(nx+1,:) = 0.
  do k=1,nzm
   do i=1,nx
    u(i,:,k) = -(psi(i,k+1)-psi(i,k))/(rho(k)*adz(k)*dz)
    w(i,:,k) = (psi(i+1,k)-psi(i,k))/(rhow(k)*dx)
   end do
  end do
  w(:,:,nz) = 0.
  v(:,:,:) = 0.

 case(2)

  ! rectangular cell (rotation along horizontal axes

  pi = acos(-1.)

  call task_rank_to_index(rank,it,jt)

  xc = dx*(nx_gl/2+1) ! center's x
  zc = zi(nz)/2. ! center's z
  psi = 0.
  do k=1,nzm+1
    if(zi(k).le.top) then
     do i=1,nx+1
       x = dx*(i+it-1)
       psi(i,k) = cos(pi/2.*(x-xc)/xc)*cos(pi/2.*(zi(k)-zc)/zc)*1.e5/5.
     end do
    end if
  end do
  if(1+it.eq.1) psi(1,:) = 0.
  if(nx+1+it.eq.nx_gl+1) psi(nx+1,:) = 0.
  do k=1,nzm
   do i=1,nx
    u(i,:,k) = -(psi(i,k+1)-psi(i,k))/(rho(k)*adz(k)*dz)
    w(i,:,k) = (psi(i+1,k)-psi(i,k))/(rhow(k)*dx)
   end do
  end do
  w(:,:,nz) = 0.
  v(:,:,:) = 0.



end select

end
